"""
Classifies: CHEBI:15705 L-alpha-amino acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_L_alpha_amino_acid(smiles: str):
    """
    Determines if a molecule is an L-alpha-amino acid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an L-alpha-amino acid, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Find alpha carbon (carbon with both NH2/NH3+ and COOH/COO-)
    alpha_carbons = []
    for atom in mol.GetAtoms():
        if atom.GetSymbol() != 'C':
            continue
            
        # Check if carbon has NH2/NH3+ group and COOH/COO- group
        has_amino = False
        has_carboxyl = False
        
        for neighbor in atom.GetNeighbors():
            # Check for NH2/NH3+ or N-substituted amino group
            if neighbor.GetSymbol() == 'N':
                n_hydrogens = neighbor.GetTotalNumHs() + (1 if neighbor.GetFormalCharge() > 0 else 0)
                if n_hydrogens >= 0:  # Allow N-substituted amino groups
                    has_amino = True
                    
            # Check for COOH/COO-
            elif neighbor.GetSymbol() == 'C':
                oxygen_count = 0
                oh_count = 0
                
                for nn in neighbor.GetNeighbors():
                    if nn.GetSymbol() == 'O':
                        oxygen_count += 1
                        if nn.GetTotalNumHs() >= 1:
                            oh_count += 1
                
                if oxygen_count == 2 and (oh_count >= 1 or neighbor.GetFormalCharge() < 0):
                    has_carboxyl = True
        
        if has_amino and has_carboxyl:
            alpha_carbons.append(atom.GetIdx())
            
    if not alpha_carbons:
        return False, "No alpha carbon with both amino and carboxyl groups found"

    # Check stereochemistry
    for ac_idx in alpha_carbons:
        atom = mol.GetAtomWithIdx(ac_idx)
        if atom.GetChiralTag() in [Chem.rdchem.ChiralType.CHI_TETRAHEDRAL_CCW, 
                                 Chem.rdchem.ChiralType.CHI_TETRAHEDRAL_CW]:
            # Check if it's really L-configuration by looking at the substituents
            neighbors = list(atom.GetNeighbors())
            
            # Find COOH group
            carboxyl_idx = -1
            amino_idx = -1
            for n in neighbors:
                if n.GetSymbol() == 'C':
                    for nn in n.GetNeighbors():
                        if nn.GetSymbol() == 'O':
                            carboxyl_idx = n.GetIdx()
                            break
                elif n.GetSymbol() == 'N':
                    amino_idx = n.GetIdx()
                    
            if carboxyl_idx != -1 and amino_idx != -1:
                return True, "Found L-alpha-amino acid structure"
            
    return False, "No L-configuration found at alpha carbon"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:15705',
                          'name': 'L-alpha-amino acid',
                          'definition': 'Any alpha-amino acid having '
                                        'L-configuration at the alpha-carbon.',
                          'parents': ['CHEBI:33704']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.43478260869565216 is too low.\n'
               'True positives: '
               "[('C[C@@H]1O[C@@H](OC[C@H]2O[C@@H](NC(=O)C[C@H](N)C(O)=O)[C@H](NC(C)=O)[C@@H](O)[C@@H]2O[C@@H]2O[C@H](CO)[C@@H](O[C@@H]3O[C@H](CO[C@H]4O[C@H](CO)[C@@H](O)[C@H](O)[C@@H]4O[C@@H]4O[C@H](CO)[C@@H](O)[C@H](O)[C@H]4NC(C)=O)[C@@H](O)[C@H](O[C@H]4O[C@H](CO)[C@@H](O)[C@H](O)[C@@H]4O[C@@H]4O[C@H](CO)[C@@H](O)[C@H](O)[C@H]4NC(C)=O)[C@@H]3O)[C@H](O)[C@H]2NC(C)=O)[C@@H](O)[C@H](O)[C@@H]1O', "
               "'Found L-alpha-amino acid structure'), "
               "('N[C@@H](CC(=C)C(O)=O)C(O)=O', 'Found L-alpha-amino acid "
               "structure (CW configuration)'), "
               "('N[C@@H](CS[C@H](\\\\C=C\\\\C=C\\\\C=C/CCCC(O)=O)[C@@H](O)CCCC(O)=O)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('C1(=CNC2=C1C=CC(=C2)Cl)C[C@@H](C(=O)O)N', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('CC[C@H](C)[C@H](NC)C(O)=O', 'Found L-alpha-amino acid "
               "structure'), ('N[C@@H](Cn1cccn1)C(O)=O', 'Found L-alpha-amino "
               "acid structure (CW configuration)'), ('CSCC[C@H](N)C(O)=O', "
               "'Found L-alpha-amino acid structure'), "
               "('N[C@@H](CCCCn1cc[n+](CCCC[C@H](N)C(O)=O)c1)C(O)=O', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('CCCCCCCC\\\\C=C/C=C/C=C/[C@@H](SC[C@H](N)C(O)=O)[C@@H](O)CCCC(O)=O', "
               "'Found L-alpha-amino acid structure'), "
               "('CC(C)(COP(O)(=O)OC[C@H](N)C(O)=O)[C@@H](O)C(=O)NCCC(=O)NCCSC([*])=O', "
               "'Found L-alpha-amino acid structure'), "
               "('C(=O)([C@@H](N)C=1C=C(C=C(C1)O)O)O', 'Found L-alpha-amino "
               "acid structure (CW configuration)'), "
               "('N1(C([C@H](C1)NC(/C(/C=2C=CC(OCC[C@@H](C(=O)O)N)=CC2)=N\\\\O)=O)=O)[C@@H](C(O)=O)C3=CC=C(C=C3)O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('NC[C@H](CC[C@H](N)C(O)=O)OP(O)(O)=O', 'Found L-alpha-amino "
               "acid structure'), "
               "('OC(=O)[C@@H](N)CC1=C(C(=C(O)C(=C1[2H])[2H])[2H])[2H]', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('NCCSC[C@H](N)C(O)=O', 'Found L-alpha-amino acid structure'), "
               "('N[C@@H](CCCCNc1nc(=N)ccn1[C@@H]1O[C@H](CO)[C@@H](O)[C@H]1O)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('C[C@](N)(CCC(N)=O)C(O)=O', 'Found L-alpha-amino acid "
               "structure'), "
               "('N[C@@H](Cc1cc(\\\\N=C\\\\Cc2ccccc2)c(O)cc1O)C(O)=O', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('N[C@H](C(=O)O)CC=1C2=C(NC1)C=CC(=C2)O', 'Found L-alpha-amino "
               "acid structure'), ('S(=O)(C[C@H](N)C(O)=O)C', 'Found "
               "L-alpha-amino acid structure'), "
               "('N[C@@H](CCC(=O)NCCC#N)C(O)=O', 'Found L-alpha-amino acid "
               "structure (CW configuration)'), "
               "('C1(=CNC2=C1C=C(C=C2)Cl)C[C@@H](C(=O)O)N', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(O)[C@@H](N)CC1CC1', 'Found L-alpha-amino acid structure "
               "(CW configuration)'), ('N[C@@H](Cc1c[nH]c2cc(Br)ccc12)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('N[C@@H](CCCCNC(O)=O)C(O)=O', 'Found L-alpha-amino acid "
               "structure (CW configuration)'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CSC[C@H](N)C(O)=O', "
               "'Found L-alpha-amino acid structure'), ('NCCCC[C@H](N)C(O)=O', "
               "'Found L-alpha-amino acid structure'), "
               "('N[C@@H](CCCCNC=O)C(O)=O', 'Found L-alpha-amino acid "
               "structure (CW configuration)'), ('NCCC[C@H](O)[C@H](N)C(O)=O', "
               "'Found L-alpha-amino acid structure'), "
               "('N[C@@H](Cc1cccc(O)c1)C(O)=O', 'Found L-alpha-amino acid "
               "structure (CW configuration)'), "
               "('COC1=CC=C(C[C@H](N)C(O)=O)C=C1O', 'Found L-alpha-amino acid "
               "structure'), ('N[C@@H](Cc1c[nH]c2c(O)cccc12)C(O)=O', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('[H][C@](N)(CS[C@H](\\\\C=C\\\\C=C\\\\C=C/C\\\\C=C/CCC(O)=O)[C@@H](O)CCCC(O)=O)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(O)[C@@H](N)CCC1C=CC(N)C=C1', 'Found L-alpha-amino acid "
               "structure (CW configuration)'), ('N[C@@H](CC1=CCC=CC1)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('NCC[C@@H](O)C[C@H](N)C(O)=O', 'Found L-alpha-amino acid "
               "structure'), ('OC(=O)[C@@H](N)CCCCN.OC(=O)CCC', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('O[C@@H]1CCN[C@@H]1C(O)=O', 'Found L-alpha-amino acid "
               "structure'), ('C([C@](N)(C)CO)(O)=O', 'Found L-alpha-amino "
               "acid structure'), "
               "('NC(=O)CC[C@H](Nc1c(cc(cc1[N+]([O-])=O)[N+]([O-])=O)[N+]([O-])=O)C(O)=O', "
               "'Found L-alpha-amino acid structure')]\n"
               'False positives: '
               "[('O([C@H]1O[C@@H]([C@@H](O[C@@H]2O[C@@H]([C@@H](O[C@@H]3O[C@@H]([C@H](O)[C@H](O[C@]4(O[C@H]([C@H](NC(=O)C)[C@@H](O)C4)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]3O)CO)[C@H](O)[C@H]2NC(=O)C)CO)[C@H](O)[C@@H]1O[C@@H]5O[C@@H]([C@@H](O[C@@H]6O[C@@H]([C@H](O)[C@H](O)[C@H]6O)CO[C@]7(O[C@H]([C@H](NC(=O)CO)[C@@H](O)C7)[C@H](O)[C@H](O)CO)C(O)=O)[C@H](O)[C@H]5NC(=O)C)CO)CO)[C@@H]8[C@H](O)[C@@H](O[C@@H]([C@H]8O)CO[C@H]9O[C@@H]([C@@H](O)[C@H](O)[C@@H]9O[C@@H]%10O[C@@H]([C@@H](O[C@@H]%11O[C@@H]([C@H](O)[C@H](O)[C@H]%11O)CO[C@]%12(O[C@H]([C@H](NC(=O)C)[C@@H](O)C%12)[C@H](O)[C@H](O)CO)C(O)=O)[C@H](O)[C@H]%10NC(=O)C)CO)CO)O[C@H]%13[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]%13CO)O[C@H]%14[C@H](O)[C@@H](NC(=O)C)C(O[C@@H]%14CO[C@@H]%15O[C@H]([C@@H](O)[C@@H](O)[C@@H]%15O)C)O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('CCCCCCCC(=O)N[C@@H](CCO)C(O)=O', 'Found L-alpha-amino acid "
               "structure (CW configuration)'), "
               "('O=C(N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@@H]1[C@H](O)[C@@H](O[C@@H]([C@H]1O)CO[C@H]2O[C@@H]([C@@H](O)[C@H](O)[C@@H]2O[C@@H]3O[C@@H]([C@@H](O[C@@H]4O[C@@H]([C@H](O)[C@H](O[C@@H]5O[C@@H]([C@H](O)[C@H](O)[C@H]5O)CO)[C@H]4O)CO)[C@H](O)[C@H]3NC(=O)C)CO)CO)O[C@H]6[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]6CO)O[C@H]7[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]7CO)O)[C@H]8O[C@@H]([C@@H](O)[C@H](O)[C@@H]8O[C@@H]9O[C@@H]([C@@H](O[C@@H]%10O[C@@H]([C@H](O)[C@H](O[C@@H]%11O[C@@H]([C@@H](O[C@@H]%12O[C@@H]([C@H](O)[C@H](O[C@@H]%13O[C@@H]([C@@H](O[C@@H]%14O[C@@H]([C@H](O)[C@H](O[C@@H]%15O[C@@H]([C@@H](O[C@@H]%16O[C@@H]([C@H](O)[C@H](O)[C@H]%16O)CO)[C@H](O)[C@H]%15NC(=O)C)CO)[C@H]%14O)CO)[C@H](O)[C@H]%13NC(=O)C)CO)[C@H]%12O)CO)[C@H](O)[C@H]%11NC(=O)C)CO)[C@H]%10O)CO)[C@H](O)[C@H]9NC(=O)C)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@@H]1O[C@@H]([C@@H](O)[C@H](O[C@@H]2O[C@@H]([C@H](O)[C@H](O[C@]3(O[C@H]([C@H](NC(=O)C)[C@@H](O)C3)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]2O)CO)[C@H]1NC(=O)C)CO[C@]4(O[C@H]([C@H](NC(=O)C)[C@@H](O)C4)[C@H](O)[C@H](O)CO)C(O)=O)[C@@H]5[C@@H](O[C@@H]6[C@H](O)[C@@H](O[C@@H]([C@H]6O)CO[C@H]7O[C@@H]([C@@H](O)[C@H](O)[C@@H]7O[C@@H]8O[C@@H]([C@@H](O[C@@H]9O[C@@H]([C@H](O)[C@H](O[C@]%10(O[C@H]([C@H](NC(=O)C)[C@@H](O)C%10)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]9O)CO)[C@H](O)[C@H]8NC(=O)C)CO)CO)O[C@H]%11[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]%11CO)O[C@H]%12[C@H](O)[C@@H](NC(=O)C)C(O[C@@H]%12CO[C@@H]%13O[C@H]([C@@H](O)[C@@H](O)[C@@H]%13O)C)O)O[C@@H]([C@@H](O)[C@@H]5O)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](CC(=O)N)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(=O)N)CCCCN', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O1[C@@H](O[C@@H]2[C@@H](O)[C@H](O[C@@H]3[C@@H](NC(=O)C)[C@H](O[C@@H]4[C@@H](O)[C@H](O[C@H]5[C@@H](O)[C@H](OC(O)[C@@H]5NC(=O)C)CO[C@@H]6O[C@@H]([C@@H](O)[C@H](O[C@@H]7O[C@@H]([C@H](O)[C@H](O)[C@H]7O)CO)[C@H]6NC(=O)C)CO)O[C@@H]([C@@H]4O)CO)O[C@@H]([C@H]3O)CO)O[C@@H]([C@@H]2O)CO)[C@H](NC(=O)C)[C@@H](O[C@@H]8O[C@@H]([C@H](O)[C@H](O)[C@H]8O[C@@H]9O[C@H]([C@@H](O)[C@@H](O)[C@@H]9O)C)CO)[C@H](O[C@@H]%10O[C@H]([C@@H](O)[C@@H](O)[C@@H]%10O)C)[C@H]1CO', "
               "'Found L-alpha-amino acid structure'), "
               "('O[C@@H]([C@H](N)C(=O)N[C@@H](CC(=O)N)C(=O)N[C@@H](CO)C(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('OC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CO)CC(O)=O)[C@H](CC)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](CCC(O)=O)C(=O)N[C@@H]([C@H](O)C)C(O)=O)[C@@H](N)C(C)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O[C@@H]([C@H](N)C(=O)N[C@@H](CCC(=O)N)C(=O)N[C@@H](C)C(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('[H][C@]12O[C@@H](SC[C@H](NC(=O)CC[C@H](N)C(O)=O)C(=O)NCC(O)=O)[C@H](O)[C@@]1([H])c1c(O2)cc(OC)c2c3CCC(=O)c3c(=O)oc12', "
               "'Found L-alpha-amino acid structure'), "
               "('O=C1N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)O)CC(=O)N[C@@H](CCC(=O)OC)C(N[C@H]([C@@H](C(N[C@H](CCC(NC1=C)=O)C(=O)O)=O)C)/C=C/C(=C/[C@@H]([C@@H](OC)CC2=CC=CC=C2)C)/C)=O)CCC(=O)OC)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('P(OC[C@H](OC(=O)CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)COCCCCCCCCCCCCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Found L-alpha-amino acid structure'), "
               "('CC(C)C[C@H](NC(=O)[C@@H](N)CC(O)=O)C(=O)N[C@@H](CC(C)C)C(=O)N[C@@H](CO)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](CC=1NC=NC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(O)=O)CCC(=O)N', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](CC(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=CC=C1)CC(C)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('S(CC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H](CC1=CC=C(O)C=C1)C(O)=O)CO)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C1N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)O)[C@@H](C(=O)N[C@@H](CC2=CC=C(O)C=C2)C(N[C@H]([C@@H](C(N[C@H](CCC(NC1=C)=O)C(=O)O)=O)C)/C=C/C(=C/[C@@H]([C@@H](OC)CC3=CC=CC=C3)C)/C)=O)C)CCCN=C(N)N)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('P(OC[C@H](OC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)CO/C=C\\\\CCCCCCCCCCCCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Found L-alpha-amino acid structure'), "
               "('S(CC[C@@H]1NC(=O)[C@H]([C@@H](NC(=O)[C@@H](NC(=O)[C@H](NC(=O)C(N(C(CC[C@@H](NC([C@H]([C@@H](NC1=O)/C=C/C(=C/[C@@H]([C@@H](OC)CC2=CC=CC=C2)C)/C)C)=O)C(=O)O)=O)C)=C)C)CC(C)C)C(=O)O)C)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C1N[C@H](C(=O)N[C@H](C(=O)N2[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N3[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)O)CC(=O)N)CCC(=O)O)CC(C)C)CCCNC(=N)N)CCCNC(=N)N)CO)CC(C)C)C(C)C)CCC3)CC=4NC=NC4)CCC(=O)O)CCC(=O)O)CO)CC(=O)N)CCC(=O)N)CCC2)CC(NCC(N[C@H]1[C@H](O)C)=O)=O)[C@H](CC)C', "
               "'Found L-alpha-amino acid structure'), "
               "('O=C(N[C@@H](CC=1NC=NC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCCCN)CCCCN', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CC=1NC=NC1)C(=O)N[C@@H](CC=2C=3C(NC2)=CC=CC3)C(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('CC(=O)N(O)CCC[C@H](N)C(=O)N[C@@H](CCCN(O)C(C)=O)C(=O)N[C@@H](CCCN(O)C(C)=O)C(=O)N[C@@H](CO)C(=O)N[C@H]([C@H](O)[C@H]1S[C@H]([C@H](O)[C@H]1O)n1ccc(=N)n(C)c1=O)C(O)=O', "
               "'Found L-alpha-amino acid structure'), "
               "('O=C(N[C@@H](CCCCN)C(O)=O)[C@@H](NC(=O)[C@@H](N)CO)CC(C)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](CC1=CC=C(O)C=C1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(O)=O)[C@H](CC)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@@H]1[C@H](O)[C@@H](O[C@@H]([C@H]1O)CO[C@H]2O[C@@H]([C@@H](O)[C@H](O)[C@@H]2O[C@@H]3O[C@@H]([C@@H](O[C@@H]4O[C@@H]([C@H](O)[C@H](O)[C@H]4O)CO[C@]5(O[C@H]([C@H](NC(=O)C)[C@@H](O)C5)[C@H](O)[C@H](O)CO)C(O)=O)[C@H](O)[C@H]3NC(=O)C)CO)CO)O[C@H]6[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]6CO)O[C@H]7[C@H](O)[C@@H](NC(=O)C)C(O[C@@H]7CO)O)[C@H]8O[C@@H]([C@@H](O)[C@H](O)[C@@H]8O[C@@H]9O[C@@H]([C@@H](O[C@@H]%10O[C@@H]([C@H](O)[C@H](O)[C@H]%10NC(=O)C)CO[C@]%11(O[C@H]([C@H](NC(=O)C)[C@@H](O)C%11)[C@H](O)[C@H](O)CO)C(O)=O)[C@H](O)[C@H]9NC(=O)C)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](CC(O)=O)C(O)=O)[C@H]1N(CCC1)C(=O)[C@@H](N)CC(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=1NC=NC1)C(C)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C1N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)O)CC(=O)N[C@@H](CCCN=C(N)N)C(N[C@H]([C@@H](C(N[C@H](CCC(N[C@H]1CO)=O)C(=O)O)=O)C)/C=C/C(=C/[C@@H]([C@@H](OC)CC2=CC=CC=C2)C)/C)=O)CC(C)C)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O1[C@@H]([C@@H](O[C@@H]2O[C@@H]([C@@H](O)[C@H](O)[C@H]2NC(=O)C)CO)[C@H](O[C@H]3O[C@@H]([C@@H](O[C@@H]4O[C@@H]([C@@H](O)[C@H](O)[C@H]4NC(=O)C)CO)[C@H](O)[C@@H]3O)CO)[C@H](O)[C@@H]1O[C@H]5[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]5CO)O[C@H]6[C@H](O)[C@@H](NC(=O)C)C(O[C@@H]6CO)O)CO[C@H]7O[C@@H]([C@@H](O)[C@H](O)[C@@H]7O[C@@H]8O[C@@H]([C@@H](O)[C@H](O)[C@H]8NC(=O)C)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](CC(=O)N)C(O)=O)[C@@H](NC(=O)[C@@H](N)CO)CCC(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](NC(=O)CN)CCC(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H]([C@H](CC)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)CC(C)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(=O)N[C@@H]([C@H](O)C)C(O)=O)[C@H]3N(CCC3)C(=O)[C@@H](N)CC4=CC=C(O)C=C4', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('CC[C@H](C)[C@H](N)C(=O)N[C@@H]([C@@H](C)CC)C(O)=O', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('P(OC[C@H](O)COC(=O)CCCCCCCCCCCCCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Found L-alpha-amino acid structure'), "
               "('O([C@@H]1[C@H](O)[C@H](O[C@H]2[C@H](O)[C@@H](NC(=O)C)[C@H](O[C@@H]2CO)O[C@H]3[C@H](O)[C@@H](NC(=O)C)[C@H](O[C@@H]3CO[C@H]4O[C@@H]([C@H](O)[C@H](O)[C@H]4O)C)O)O[C@@H]([C@H]1O)CO[C@H]5O[C@@H]([C@@H](O)[C@H](O[C@H]6O[C@@H]([C@@H](O)[C@H](O)[C@@H]6O)CO)[C@@H]5O)CO)[C@H]7O[C@@H]([C@@H](O)[C@H](O)[C@@H]7O[C@@H]8O[C@@H]([C@@H](O[C@@H]9O[C@@H]([C@H](O)[C@H](O[C@]%10(O[C@H]([C@H](NC(=O)C)[C@@H](O)C%10)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]9O)CO)[C@H](O)[C@H]8NC(=O)C)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@H]1[C@H](O[C@@H]2O[C@H]([C@@H](O)[C@@H](O)[C@@H]2O)C)[C@@H](NC(=O)C)[C@@H](O[C@@H]1CO)OC[C@H]3O[C@H](O)[C@H](NC(=O)C)[C@@H](O[C@@H]4O[C@@H]([C@H](O)[C@H](O[C@]5(O[C@H]([C@H](NC(=O)C)[C@@H](O)C5)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]4O)CO)[C@H]3O)[C@@H]6O[C@@H]([C@H](O)[C@H](O[C@]7(O[C@H]([C@H](NC(=O)C)[C@@H](O)C7)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]6O)CO', "
               "'Found L-alpha-amino acid structure'), "
               "('S(CC[C@H](NC(=O)CNC(=O)[C@@H](N)CCC(O)=O)C(O)=O)C', 'Found "
               "L-alpha-amino acid structure'), "
               "('SC[C@H](NC(=O)[C@@H](N)C)C(=O)N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O1C(OC[C@H]2O[C@H](O)[C@H](NC(=O)C)[C@@H](OC3O[C@@H]([C@H](O)[C@H](O)[C@H]3O)CO)[C@H]2O)[C@H](NC(=O)C)[C@@H](O)[C@H](OC4O[C@@H]([C@H](O)[C@H](O)[C@H]4O)CO)[C@H]1CO', "
               "'Found L-alpha-amino acid structure'), "
               "('O([C@H]1O[C@@H]([C@@H](O[C@@H]2O[C@@H]([C@@H](O[C@@H]3O[C@@H]([C@H](O)[C@H](O[C@@H]4O[C@@H]([C@@H](O[C@@H]5O[C@@H]([C@H](O)[C@H](O)[C@H]5O)CO)[C@H](O)[C@H]4NC(=O)C)CO)[C@H]3O)CO)[C@H](O)[C@H]2NC(=O)C)CO)[C@H](O)[C@@H]1O[C@@H]6O[C@@H]([C@@H](O[C@@H]7O[C@@H]([C@H](O)[C@H](O)[C@H]7O)CO)[C@H](O)[C@H]6NC(=O)C)CO)CO)[C@H]8[C@H](O)[C@H](O[C@@H](O[C@H]9[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]9CO)O[C@H]%10[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]%10CO)O)[C@H]8O)CO[C@H]%11O[C@@H]([C@@H](O)[C@H](O)[C@@H]%11O[C@@H]%12O[C@@H]([C@@H](O[C@@H]%13O[C@@H]([C@H](O)[C@H](O)[C@@H]%13O)CO)[C@H](O)[C@H]%12NC(=O)C)CO)CO[C@@H]%14O[C@@H]([C@@H](O[C@@H]%15O[C@@H]([C@H](O)[C@H](O[C@@H]%16O[C@@H]([C@@H](O[C@@H]%17O[C@@H]([C@H](O)[C@H](O)[C@H]%17O)CO)[C@H](O)[C@H]%16NC(=O)C)CO)[C@H]%15O)CO)[C@H](O)[C@H]%14NC(=O)C)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('SC[C@H](N)C(=O)N[C@@H]([C@H](O)C)C(=O)N[C@@H](CCC(O)=O)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('CN[C@H](CC(C)C)C(=O)N[C@@H]1[C@H](O)c2ccc(Oc3cc4cc(Oc5ccc(cc5Cl)[C@@H](O[C@H]5C[C@](C)(N)[C@@H](O)[C@H](C)O5)[C@@H]5NC(=O)[C@H](NC(=O)[C@@H]4NC(=O)[C@H](CC(N)=O)NC1=O)c1ccc(O)c(c1)-c1c(O)cc(O)cc1[C@H](NC5=O)C(O)=O)c3O[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O[C@H]1C[C@](C)(NCc3ccc(cc3)-c3ccc(Cl)cc3)[C@@H](O)[C@H](C)O1)c(Cl)c2', "
               "'Found L-alpha-amino acid structure'), "
               "('OC(=O)[C@@H](NCC(O)=O)C', 'Found L-alpha-amino acid "
               "structure (CW configuration)'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CCSC)C(=O)N[C@@H](C(C)C)C(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O[C@@H]([C@H](NC(=O)[C@@H](N)CC=1C=2C(NC1)=CC=CC2)C(=O)N[C@@H](CC(=O)N)C(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@@H]1[C@@H](O)[C@H](O[C@H]2[C@@H](O)[C@H](OC(O)[C@@H]2NC(=O)C)CO[C@@H]3O[C@@H]([C@@H](O[C@@H]4O[C@@H]([C@H](O)[C@H](O)[C@H]4O)CO)[C@H](O[C@@H]5O[C@H]([C@@H](O)[C@@H](O)[C@@H]5O)C)[C@H]3NC(=O)C)CO)O[C@@H]([C@@H]1O)CO)[C@]6(O[C@H]([C@H](NC(=O)C)[C@@H](O)C6)[C@H](O)[C@H](O)CO)C(O)=O', "
               "'Found L-alpha-amino acid structure'), "
               "('SC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H](CC(=O)N)C(O)=O)C', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('CN(C(=O)N[C@@H]1[C@H]([C@@H]([C@H](OC1O)CO)O)O)[NH2+][O-]', "
               "'Found L-alpha-amino acid structure'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=C(O)C=C1)CC(O)=O)C(O)=O)C', "
               "'Found L-alpha-amino acid structure'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSCC(=O)SC[C@H](NC(=O)CC[C@H](N)C(O)=O)C(=O)NCC(O)=O)C(=O)NCC(O)=O)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C1N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)O)CC(=O)N[C@@H](CCCNC(=O)N)C(N[C@H]([C@@H](C(N[C@H](CCC(N(C1=C)C)=O)C(=O)O)=O)C)/C=C/C(=C/[C@@H]([C@@H](OC)CC2=CC=CC=C2)C)/C)=O)CCCN=C(N)N)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@H](C(=O)N([C@H](C(=O)N[C@H](C(=O)O)CC1=CC=C(OCC=C(C)C)C=C1)[C@H](CC)C)C)CO)[C@H]2N(C(=O)C(O)CC3=CC=CC=C3)CCC2', "
               "'Found L-alpha-amino acid structure'), "
               "('CC(=O)N[C@H]1[C@@H](O)O[C@H](CO[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O[C@@H]3O[C@H](CO)[C@H](O)[C@H](O)[C@H]3O)[C@H]2NC(C)=O)[C@H](O)[C@@H]1O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@H]1[C@H](O)[C@@H](NC(=O)C)C(O[C@@H]1CO)O)[C@@H]2O[C@@H]([C@H](O)[C@H](O[C@@H]3O[C@@H]([C@H](O)[C@H](O)[C@H]3O)CO)[C@H]2O)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('CC(=O)N[C@H]1[C@H](O)O[C@H](CO[C@@H]2O[C@H](CO)[C@@H](O[C@@H]3O[C@H](CO)[C@H](O)[C@H](O)[C@H]3O)[C@H](O)[C@H]2O)[C@@H](O)[C@@H]1O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@@H]1[C@H](O)[C@H](O[C@H]2[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]2CO)O[C@H]3[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]3CO[C@@H]4O[C@H]([C@@H](O)[C@@H](O)[C@@H]4O)C)O)O[C@@H]([C@H]1O)CO[C@H]5O[C@@H]([C@@H](O)[C@H](O)[C@@H]5O[C@@H]6O[C@@H]([C@@H](O[C@@H]7O[C@@H]([C@H](O)[C@H](O)[C@H]7O)CO[C@]8(O[C@H]([C@H](NC(=O)C)[C@@H](O)C8)[C@H](O)[C@H](O)CO)C(O)=O)[C@H](O)[C@H]6NC(=O)C)CO)CO)[C@H]9O[C@@H]([C@@H](O)[C@H](O)[C@@H]9O[C@@H]%10O[C@@H]([C@@H](O[C@@H]%11O[C@@H]([C@H](O)[C@H](O[C@]%12(O[C@H]([C@H](NC(=O)C)[C@@H](O)C%12)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]%11O)CO)[C@H](O)[C@H]%10NC(=O)C)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CS)C(=O)N[C@@H](CC(=O)N)C(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@H]1[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]1CO)O[C@H]2[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]2CO[C@@H]3O[C@H]([C@@H](O)[C@@H](O)[C@@H]3O)C)O)[C@@H]4O[C@@H]([C@@H](O)[C@H](O)[C@@H]4O[C@H]5O[C@@H]([C@@H](O)[C@H](O)[C@@H]5O[C@@H]6O[C@@H]([C@@H](O[C@@H]7O[C@@H]([C@H](O)[C@H](O)[C@H]7O)CO[C@]8(O[C@H]([C@H](NC(=O)C)[C@@H](O)C8)[C@H](O)[C@H](O)CO)C(O)=O)[C@H](O)[C@H]6NC(=O)C)CO)CO)CO[C@H]9O[C@@H]([C@@H](O)[C@H](O)[C@@H]9O[C@@H]%10O[C@@H]([C@@H](O[C@@H]%11O[C@@H]([C@H](O)[C@H](O)[C@H]%11O)CO)[C@H](O[C@@H]%12O[C@H]([C@@H](O)[C@@H](O)[C@@H]%12O)C)[C@H]%10NC(=O)C)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@H](C(=O)N[C@@H](C)C(O)=O)C)[C@H]1NCCC1', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('O[C@@H]([C@H](NC(=O)[C@@H](N)[C@H](O)C)C(=O)N[C@@H](CCC(O)=O)C(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('[H][C@]1(O[C@@](C[C@H](O)[C@H]1NC(C)=O)(O[C@H]1[C@@H](O)[C@@H](CO)O[C@@H](O[C@H]2[C@H](O)[C@@H](NC(C)=O)[C@H](OC[C@H]3OC(O)[C@H](NC(C)=O)[C@@H](O[C@@H]4O[C@H](CO)[C@H](O)[C@H](O)[C@H]4O)[C@H]3O)O[C@@H]2CO)[C@@H]1O)C(O)=O)[C@H](O)[C@H](O)CO', "
               "'Found L-alpha-amino acid structure'), "
               "('O([C@@H]1[C@H](O)[C@@H](O[C@@H]([C@H]1O)CO[C@H]2O[C@@H]([C@@H](O)[C@H](O[C@H]3O[C@@H]([C@@H](O)[C@H](O)[C@@H]3O)CO)[C@@H]2O)CO[C@H]4O[C@@H]([C@@H](O)[C@H](O[C@H]5O[C@@H]([C@@H](O)[C@H](O)[C@@H]5O)CO)[C@@H]4O)CO)O[C@H]6[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]6CO)O[C@H]7[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]7CO)O)[C@H]8O[C@@H]([C@@H](O)[C@H](O[C@H]9O[C@@H]([C@@H](O)[C@H](O)[C@@H]9O)CO)[C@@H]8O[C@H]%10O[C@@H]([C@@H](O)[C@H](O)[C@@H]%10O[C@H]%11O[C@@H]([C@@H](O)[C@H](O)[C@@H]%11O[C@H]%12O[C@@H]([C@@H](O)[C@H](O)[C@@H]%12O)CO)CO)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CCCCN)CC=1NC=NC1)C(O)=O)C', "
               "'Found L-alpha-amino acid structure'), "
               "('SC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC=1NC=NC1)C(O)=O)[C@H]2NCCC2', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('SC[C@H](NC(=O)CN)C(=O)N[C@@H]([C@H](CC)C)C(O)=O', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('S1[C@H]([C@H]2NC(=O)[C@@H](NC(=O)CNC(=O)[C@@H](NC(=O)[C@H]3NC(=O)[C@@H](NC(=O)[C@H]([C@H](CC)C)NC([C@H]4[C@@H](SC[C@H](NC([C@H](NC([C@@H](NC([C@@H](NC([C@H](C1)NC(=O)[C@@H]([N+](C)(C)C)CCC(=O)O)=O)C)=O)CO)=O)[C@@H](SC3)C)=O)C(N[C@@H](CNCCCC[C@H](NC2=O)C(=O)O)C(=O)N[C@H](C(=O)NCC(=O)N[C@H](C(=O)N[C@H](C(N4)=O)C(C)C)[C@H](CC)C)CC5=CC=CC=C5)=O)C)=O)C(C)C)[C@@H](O)C(=O)O)[C@H](O)C)C', "
               "'Found L-alpha-amino acid structure'), "
               "('SC[C@H](N)C(=O)N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(=O)N[C@@H]([C@H](CC)C)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@@H]1[C@H](O)[C@H](O[C@H]2[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]2CO)O[C@H]3[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]3CO)O)O[C@@H]([C@H]1O)CO[C@H]4O[C@@H]([C@@H](O)[C@H](O[C@H]5O[C@@H]([C@@H](O)[C@H](O)[C@@H]5O[C@H]6O[C@@H]([C@@H](O)[C@H](O)[C@@H]6O)CO)CO)[C@@H]4O)CO[C@H]7O[C@@H]([C@@H](O)[C@H](O)[C@@H]7O[C@H]8O[C@@H]([C@@H](O)[C@H](O)[C@@H]8O)CO)CO)[C@H]9O[C@@H]([C@@H](O)[C@H](O)[C@@H]9O[C@H]%10O[C@@H]([C@@H](O)[C@H](O)[C@@H]%10O[C@H]%11O[C@@H]([C@@H](O)[C@H](O[C@H]%12O[C@@H]([C@@H](O)[C@H](O[C@H]%13O[C@@H]([C@@H](O)[C@H](O)[C@H]%13O[C@H]%14O[C@@H]([C@@H](O)[C@H](O)[C@H]%14O)CO)CO)[C@H]%12O)CO)[C@@H]%11O)CO)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@@H]1[C@@H](NC(=O)C)C(O[C@@H]([C@H]1O)CO)OC[C@H]2O[C@H](O)[C@H](NC(=O)C)[C@@H](OC3O[C@@H]([C@H](O)[C@H](O)[C@H]3O)CO)[C@H]2O)C4O[C@@H]([C@H](O)[C@H](O)[C@H]4OC5O[C@H]([C@@H](O)[C@@H](O)[C@@H]5O)C)CO', "
               "'Found L-alpha-amino acid structure'), "
               "('O([C@H]1[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]1CO)O[C@H]2[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]2CO[C@H]3O[C@@H]([C@H](O)[C@H](O)[C@H]3O)C)O)[C@@H]4O[C@@H]([C@@H](O)[C@H](O[C@H]5O[C@@H]([C@@H](O)[C@H](O)[C@@H]5O[C@@H]6O[C@@H]([C@@H](O[C@@H]7O[C@@H]([C@H](O)[C@H](O)[C@H]7O)CO)[C@H](O)[C@H]6NC(=O)C)CO)CO)[C@@H]4O)CO[C@H]8O[C@@H]([C@@H](O)[C@H](O[C@H]9O[C@@H]([C@@H](O)[C@H](O)[C@@H]9O[C@H]%10O[C@@H]([C@@H](O)[C@H](O)[C@@H]%10O)CO)CO)[C@@H]8O)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('S(CC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H]([C@H](CC)C)C(O)=O)CC(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('N[C@@H](CSC1=CC(=O)C(=O)c2[nH]cc(C[C@H](N)C(O)=O)c12)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@H]1[C@H](O[C@@H]2O[C@@H]([C@@H](O)[C@H](O)[C@H]2NC(=O)C)CO)[C@H](O[C@@H](O[C@H]3[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]3CO)O[C@H]4[C@H](O)[C@@H](NC(=O)C)C(O[C@@H]4CO[C@@H]5O[C@H]([C@@H](O)[C@@H](O)[C@@H]5O)C)O)[C@H]1O)CO[C@H]6O[C@@H]([C@@H](O)[C@H](O[C@H]7O[C@@H]([C@@H](O)[C@H](O)[C@@H]7O)CO)[C@@H]6O)CO[C@H]8O[C@@H]([C@@H](O)[C@H](O)[C@@H]8O)CO)[C@H]9O[C@@H]([C@@H](O[C@@H]%10O[C@@H]([C@@H](O[C@@H]%11O[C@@H]([C@H](O)[C@H](O)[C@H]%11O)CO)[C@H](O)[C@H]%10NC(=O)C)CO)[C@H](O)[C@@H]9O[C@@H]%12O[C@@H]([C@@H](O[C@@H]%13O[C@@H]([C@H](O)[C@H](O)[C@H]%13O)CO)[C@H](O)[C@H]%12NC(=O)C)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H]([C@H](O)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=CC=C1)CC(C)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@H]1[C@H](O[C@@H]2O[C@H]([C@@H](O)[C@@H](O)[C@@H]2O)C)[C@@H](NC(=O)C)[C@@H](O[C@@H]1CO)O[C@@H]3[C@@H](O)[C@@H](O[C@@H]([C@@H]3O)CO)O[C@H]4[C@H](O)[C@@H](NC(=O)C)C(O[C@@H]4CO)O)[C@@H]5O[C@@H]([C@H](O)[C@H](O[C@]6(O[C@H]([C@H](NC(=O)C)[C@@H](O)C6)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]5O)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('N[C@@H](COP(O)(=O)O[C@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@@H]1O)C(O)=O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('S(CC[C@H](NC(=O)CN)C(=O)N[C@@H](CCCCN)C(O)=O)C', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('S(C[C@@H](NC(=O)C)C(O)=O)C/C=C/CO', 'Found L-alpha-amino "
               "acid structure (CW configuration)'), "
               "('S(CC[C@H](N)C(=O)N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(=O)N[C@@H](CC3=CC=CC=C3)C(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](C(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)C)CC=1C=2C(NC1)=CC=CC2', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@@H]1[C@H](O)[C@H](O[C@H]2[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]2CO)O[C@H]3[C@H](O)[C@@H](NC(=O)C)[C@H](O[C@@H]3CO[C@@H]4O[C@@H]([C@H](O)[C@H](O)[C@H]4O)C)O)O[C@@H]([C@H]1O)CO[C@H]5O[C@@H]([C@@H](O)[C@H](O)[C@@H]5O)CO)[C@H]6O[C@@H]([C@@H](O)[C@H](O)[C@@H]6O[C@@H]7O[C@@H]([C@@H](O[C@@H]8O[C@@H]([C@H](O)[C@H](O[C@]9(O[C@H]([C@H](NC(=O)C)[C@@H](O)C9)[C@H](O)[C@H](O)CO)C(O)=O)[C@H]8O)CO)[C@H](O)[C@H]7NC(=O)C)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('OC(=O)[C@@H](NC(=O)CNC(=O)[C@@H](N)C(C)C)CC(C)C', 'Found "
               "L-alpha-amino acid structure (CW configuration)'), "
               "('C(=O)([C@@H](N)CCSC)N[C@H](C(=O)N[C@H](C(=O)O)CC(O)=O)CC1=CNC2=C1C=CC=C2', "
               "'Found L-alpha-amino acid structure'), "
               "('C[C@@H](O)[C@H](NC(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc2c(N)ncnc12)C(O)=O', "
               "'Found L-alpha-amino acid structure'), "
               "('O([C@H]1[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]1CO)O[C@H]2[C@H](O)[C@@H](NC(=O)C)C(O[C@@H]2CO)O)[C@@H]3O[C@@H]([C@@H](O)[C@H](O[C@H]4O[C@@H]([C@@H](O)[C@H](O[C@H]5O[C@@H]([C@@H](O)[C@H](O)[C@@H]5O)CO)[C@@H]4O)CO[C@H]6O[C@@H]([C@@H](O)[C@H](O)[C@@H]6O)CO)[C@@H]3O)CO[C@H]7O[C@@H]([C@@H](O)[C@H](O)[C@@H]7O[C@H]8O[C@@H]([C@@H](O)[C@H](O)[C@@H]8O)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@H]1[C@H](O)[C@@H](NC(=O)C)[C@@H](O[C@@H]1CO)O[C@H]2[C@H](O)[C@@H](NC(=O)C)C(O[C@@H]2CO[C@@H]3O[C@H]([C@@H](O)[C@@H](O)[C@@H]3O)C)O)[C@@H]4O[C@@H]([C@@H](O)[C@H](O[C@H]5O[C@@H]([C@@H](O)[C@H](O)[C@@H]5O[C@@H]6O[C@@H]([C@@H](O)[C@H](O)[C@@H]6NC(=O)C)CO)CO)[C@@H]4O)CO[C@H]7O[C@@H]([C@@H](O)[C@H](O)[C@@H]7O[C@@H]8O[C@@H]([C@@H](O[C@@H]9O[C@@H]([C@H](O)[C@H](O)[C@H]9NC(=O)C)CO)[C@H](O)[C@H]8NC(=O)C)CO)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O([C@H]1[C@@H](O)[C@H](O[C@@H](O[C@@H]2[C@@H](NC(=O)C)C(O[C@@H]([C@@H]2O)CO[C@@H]3O[C@@H]([C@@H](O)[C@H](O)[C@H]3NC(=O)C)CO)O)[C@@H]1O)CO[C@@H]4O[C@@H]([C@@H](O[C@@H]5O[C@@H]([C@H](O)[C@H](O)[C@H]5O[C@@H]6O[C@H]([C@@H](O)[C@@H](O)[C@@H]6O)C)CO)[C@H](O)[C@H]4NC(=O)C)CO)[C@@H]7O[C@@H]([C@@H](O)[C@H](O[C@@H]8O[C@@H]([C@H](O)[C@H](O)[C@H]8O[C@@H]9O[C@H]([C@@H](O)[C@@H](O)[C@@H]9O)C)CO)[C@H]7NC(=O)C)CO', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](C(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)C)CC1=CC=CC=C1', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('C[C@@H]1O[C@@H](O[C@H]2[C@H](O)[C@H](O)[C@H](C)O[C@H]2O[C@@H]2[C@@H](NC(C)=O)C(O)O[C@H](CO)[C@H]2O[C@@H]2O[C@H](CO)[C@H](O)[C@H](O)[C@H]2NC(C)=O)[C@@H](O)[C@H](O)[C@@H]1O', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(N[C@@H](C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CO)CC=1C=2C(NC1)=CC=CC2', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CO)C(=O)N[C@@H](CC=1NC=NC1)C(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('S(CC[C@H](N)C(=O)N1[C@@H](CCC1)C(=O)N[C@@H](CC(C)C)C(O)=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('OC1[C@@H]([C@H]([C@@H]([C@H](O1)COS([O-])(=O)=O)O[C@H]2[C@@H]([C@H]([C@H]([C@H](O2)CO)O[C@]3(O[C@]([C@@H]([C@H](C3)O)NC(C)=O)([C@@H]([C@H](O)CO)O)[H])C([O-])=O)O)O)O)NC(=O)C', "
               "'Found L-alpha-amino acid structure (CW configuration)'), "
               "('O=C(O)[C@@H](N)CC1=CC(=C(O)C=C1)C=O', 'Found L-alpha-amino "
               "acid structure (CW configuration)'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CC(O)=O)CC=1NC=NC1)C(O)=O)C', "
               "'Found L-alpha-amino acid structure')]\n"
               "False negatives: [('OC(=O)[C@@H](N(C)C)CCCCN', 'No alpha "
               "carbon with both amino and carboxyl groups found'), "
               "('C(N(C(=O)NC)O)CC[C@@H](C(=O)[O-])[NH3+]', 'No alpha carbon "
               "with both amino and carboxyl groups found'), "
               "('O=C([O-])[C@@H]([NH3+])C[C@@H](CNC(=[NH2+])N)O', 'No alpha "
               "carbon with both amino and carboxyl groups found'), "
               "('OC(=O)[C@](N([2H])[2H])(C(C(CN=C(N)N)([2H])[2H])([2H])[2H])[2H]', "
               "'No alpha carbon with both amino and carboxyl groups found')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 42,
    'num_false_positives': 100,
    'num_true_negatives': 911,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.29577464788732394,
    'recall': 0.9545454545454546,
    'f1': 0.45161290322580644,
    'accuracy': 0.9033175355450237}


"""
Classifies: CHEBI:138741 very long-chain primary fatty alcohol
"""
from rdkit import Chem
from rdkit.Chem import Descriptors

def is_very_long_chain_primary_fatty_alcohol(smiles: str):
    """
    Determines if a molecule is a very long-chain primary fatty alcohol (C23-C27).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a very long-chain primary fatty alcohol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains an -OH group
    if not any(atom.GetSymbol() == 'O' and sum(atom.GetNeighbors()) == 1 for atom in mol.GetAtoms()):
        return False, "Molecule does not contain a primary alcohol group"

    # Get the number of carbon atoms in the longest chain
    chain_length = Descriptors.GetLongestChain(mol)

    if chain_length < 23 or chain_length > 27:
        return False, f"Chain length is {chain_length}, not between C23 and C27"

    return True, f"Very long-chain primary fatty alcohol with {chain_length} carbon atoms"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:138741',
                          'name': 'very long-chain primary fatty alcohol',
                          'definition': 'Any primary fatty alcohol with a '
                                        'chain length between C23 and C27.',
                          'parents': ['CHEBI:142622']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "unsupported operand type(s) for +: 'int' and 'Atom'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:24279 glucosinolate
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Draw import IPythonConsole

def is_glucosinolate(smiles: str):
    """
    Determines if a molecule is a glucosinolate based on structural requirements:
    - Contains thioglucoside core
    - Has a central C atom bonded to S, N, and a side group
    - Contains sulfonated oxime group
    - Anti stereochemistry across C=N double bond (where specified)
    
    Args:
        smiles (str): SMILES string of molecule
        
    Returns:
        bool: True if molecule is a glucosinolate, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS patterns for key structural features
    thioglucoside = "[OX2H1][CH]1[CH]([OH])[CH]([OH])[CH]([OH])[CH]([OH])[OX2][CH]1[SX2]" # Sugar-S pattern
    sulfonated_oxime = "[NX2]=[CX3][SX2]-[OX2]-[NX2]-[SX4](=[OX1])(=[OX1])[OX1,OX2-]" # C=N-O-S(=O)(=O)[O-] pattern
    
    # Check for thioglucoside core
    if not mol.HasSubstructMatch(Chem.MolFromSmarts(thioglucoside)):
        return False, "Missing thioglucoside core structure"
        
    # Check for sulfonated oxime group
    if not mol.HasSubstructMatch(Chem.MolFromSmarts(sulfonated_oxime)):
        return False, "Missing sulfonated oxime group"

    # Find central carbon with S, N bonds
    central_c_pattern = "[CX3]([SX2])([NX2]=*)*" # C bonded to S, N=, and something else
    matches = mol.GetSubstructMatches(Chem.MolFromSmarts(central_c_pattern))
    
    if not matches:
        return False, "Missing central carbon with required connectivity"

    # Check stereochemistry where specified
    for match in matches:
        central_c = mol.GetAtomWithIdx(match[0])
        # Get connected atoms
        neighbors = [x.GetIdx() for x in central_c.GetNeighbors()]
        
        # If stereochemistry is specified, should be E/Z (anti)
        bonds = mol.GetBonds()
        double_bonds = [b for b in bonds if b.GetBondType() == Chem.BondType.DOUBLE and 
                       (b.GetBeginAtomIdx() == match[0] or b.GetEndAtomIdx() == match[0])]
        
        if double_bonds:
            if double_bonds[0].GetStereo() == Chem.BondStereo.STEREOE:
                return True, "Valid glucosinolate with E (anti) stereochemistry"
            elif double_bonds[0].GetStereo() == Chem.BondStereo.STEREOZ:
                return False, "Invalid Z (syn) stereochemistry across C=N"
    
    # If we get here, basic structure is present but stereochemistry not specified
    return True, "Valid glucosinolate structure (stereochemistry not fully specified)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24279',
                          'name': 'glucosinolate',
                          'definition': 'Water-soluble anionic substituted '
                                        'thioglucosides. Glucosinolates have a '
                                        'central C atom which is bonded via an '
                                        'S atom to a glycone group and via an '
                                        'N atom to a sulfonated oxime group, '
                                        'and which also carries a side-group. '
                                        'The side-chain and sulfate group have '
                                        'an anti stereochemical configuration '
                                        'across the C=N double bond.',
                          'parents': ['CHEBI:24401']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183873,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999673698464752}


"""
Classifies: CHEBI:25706 organomercury compound
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_organomercury_compound(smiles: str):
    """
    Determines if a molecule is an organomercury compound (contains at least one C-Hg bond)

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an organomercury compound, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check if molecule contains mercury
    if not any(atom.GetSymbol() == 'Hg' for atom in mol.GetAtoms()):
        return False, "No mercury atoms found"

    # Find all mercury atoms
    hg_atoms = [atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'Hg']

    # Check for C-Hg bonds
    c_hg_bonds = []
    for hg_atom in hg_atoms:
        for neighbor in hg_atom.GetNeighbors():
            if neighbor.GetSymbol() == 'C':
                bond = mol.GetBondBetweenAtoms(hg_atom.GetIdx(), neighbor.GetIdx())
                c_hg_bonds.append(bond)

    if not c_hg_bonds:
        return False, "No carbon-mercury bonds found"

    # Count number of C-Hg bonds
    num_c_hg_bonds = len(c_hg_bonds)
    
    # Get bond types
    bond_types = [str(bond.GetBondType()) for bond in c_hg_bonds]
    bond_desc = ", ".join(bond_types)

    return True, f"Found {num_c_hg_bonds} carbon-mercury bond(s) of type(s): {bond_desc}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25706',
                          'name': 'organomercury compound',
                          'definition': 'A compound containing at least one '
                                        'carbon-mercury bond.',
                          'parents': ['CHEBI:25196', 'CHEBI:25707']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 11,
    'num_true_negatives': 183897,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.15384615384615385,
    'recall': 1.0,
    'f1': 0.2666666666666667,
    'accuracy': 0.999940188135501}


"""
Classifies: CHEBI:23775 dihydroxy-5beta-cholanic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_dihydroxy_5beta_cholanic_acid(smiles: str):
    """
    Determines if a molecule is a dihydroxy-5beta-cholanic acid, which is defined as a hydroxy-5beta-cholanic acid
    carrying two hydroxy groups at unspecified positions.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a dihydroxy-5beta-cholanic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule is a cholanic acid derivative
    is_cholanic_acid = AllChem.MolToSmiles(AllChem.RemoveHs(mol)) == 'C(C1CCC2C3CC(CC12)C4(C(CC3)CC(O)=O)CCC5(C)C(O)CCC45)C'
    if not is_cholanic_acid:
        return False, "Not a cholanic acid derivative"

    # Check if it has exactly two hydroxyl groups
    hydroxy_count = sum(atom.GetTotalNumHs(onlyExplicit=False) for atom in mol.GetAtoms())
    if hydroxy_count != 2:
        return False, "Does not have exactly two hydroxy groups"

    # Check if it has a 5-beta configuration
    ring_atoms = mol.GetRingInfo().AtomRings()[0]
    junc_atom = mol.GetAtomWithIdx(ring_atoms[5])
    neighbors = [mol.GetAtomWithIdx(n) for n in junc_atom.GetNeighbors()]
    methyl_groups = sum(atom.GetTotalNumHs(onlyExplicit=False) == 9 for atom in neighbors)
    if methyl_groups != 1:
        return False, "Does not have a 5-beta configuration"

    return True, "Molecule is a dihydroxy-5beta-cholanic acid"

# Example usage
smiles = "[H][C@]12CC[C@@]3([H])[C@]4([H])CC[C@]([H])([C@H](C)CCC(O)=O)[C@@]4(C)[C@@H](O)C[C@]3([H])[C@@]1(C)CC[C@@H](O)C2"
result, reason = is_dihydroxy_5beta_cholanic_acid(smiles)
print(result, reason)


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23775',
                          'name': 'dihydroxy-5beta-cholanic acid',
                          'definition': 'A hydroxy-5beta-cholanic acid '
                                        'carrying two hydroxy groups at '
                                        'unspecified positions.',
                          'parents': ['CHEBI:24663', 'CHEBI:85184']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "Can't kekulize mol.  Unkekulized atoms: 0 1 2 3 4 5 6 7 9 10 11 "
             '13 14 15 16 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 '
             '35 36 37 38 39 41 42 44 45 46 47 48 50 51 53 54 55 56 57 58 59 '
             '60 61 69 70 71 72 73 74 75 76 78 80 81 82 83 88 89',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:19254 purine 2'-deoxyribonucleoside
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_purine_2__deoxyribonucleoside(smiles: str):
    """
    Determines if a molecule is a purine 2'-deoxyribonucleoside.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a purine 2'-deoxyribonucleoside, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for 2'-deoxyribose moiety
    deoxyribose_pattern = Chem.MolFromSmarts("[CH2]1[CH]([OH])[CH]([CH2][OH])[O][CH]1")
    if not mol.HasSubstructMatch(deoxyribose_pattern):
        return False, "No 2'-deoxyribose moiety found"

    # Check for purine core structure with N-glycosidic bond to deoxyribose
    purine_pattern1 = Chem.MolFromSmarts("[CH2]1[CH]([OH])[CH]([CH2][OH])[O][CH]1n2cnc3c2ncnc3") # Adenine core
    purine_pattern2 = Chem.MolFromSmarts("[CH2]1[CH]([OH])[CH]([CH2][OH])[O][CH]1n2cnc3c2nc[nH]c3=O") # Guanine/inosine core
    
    if mol.HasSubstructMatch(purine_pattern1):
        return True, "Purine 2'-deoxyribonucleoside with adenine core"
    elif mol.HasSubstructMatch(purine_pattern2):
        return True, "Purine 2'-deoxyribonucleoside with guanine/inosine core"
    else:
        return False, "No purine core with correct N-glycosidic linkage found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:19254',
                          'name': "purine 2'-deoxyribonucleoside",
                          'definition': "A 2'-deoxyribonucleoside that has a "
                                        'purine moiety as the nucleobase (the '
                                        'R group in the illustration).',
                          'parents': ['CHEBI:18274', 'CHEBI:60173']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               'False negatives: '
               "[('C[C@@H](Nc1nc2n(cnc2c(=O)[nH]1)[C@H]1C[C@H](O)[C@@H](CO)O1)C(O)=O', "
               "'No purine core structure found'), "
               "('OC[C@H]1O[C@H](C[C@@H]1O)n1cnc2c1nc[nH]c2=O', 'No purine "
               "core structure found')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 11,
    'num_true_negatives': 183898,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.15384615384615385,
    'recall': 1.0,
    'f1': 0.2666666666666667,
    'accuracy': 0.9999401884607229}


"""
Classifies: CHEBI:22332 alkylamino group
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_alkylamino_group(smiles: str):
    """
    Determines if a molecule contains an alkylamino group (alkyl substituents attached via nitrogen).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains alkylamino group, False otherwise
        str: Reason for classification
    """
    # Replace wildcard atom (*) with carbon to allow RDKit processing
    smiles = smiles.replace('*', 'C')
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Find nitrogen atoms
    n_atoms = [atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'N']
    if not n_atoms:
        return False, "No nitrogen atoms found"
        
    for n_atom in n_atoms:
        # Get neighbors of nitrogen
        neighbors = n_atom.GetNeighbors()
        
        # Check if nitrogen is bonded to at least one carbon
        carbon_neighbors = [neigh for neigh in neighbors if neigh.GetSymbol() == 'C']
        if not carbon_neighbors:
            continue
            
        # Check if at least one carbon neighbor is sp3 (alkyl)
        for c_atom in carbon_neighbors:
            if c_atom.GetHybridization() == Chem.HybridizationType.SP3:
                # Check if this carbon is part of an alkyl chain
                is_alkyl = True
                for next_neigh in c_atom.GetNeighbors():
                    if next_neigh.GetSymbol() not in ['C', 'H', 'N']:
                        if not (next_neigh.GetSymbol() == 'O' and len([b for b in next_neigh.GetBonds() if b.GetBondType() == Chem.BondType.DOUBLE]) > 0):
                            is_alkyl = False
                            break
                
                if is_alkyl:
                    return True, "Contains alkylamino group - nitrogen bonded to sp3 carbon"
                    
    return False, "No alkylamino groups found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22332',
                          'name': 'alkylamino group',
                          'definition': 'Alkyl substituents attached to the '
                                        'remainder of a molecule via nitrogen.',
                          'parents': ['CHEBI:33456']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 6,
    'num_false_positives': 100,
    'num_true_negatives': 68,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.05660377358490566,
    'recall': 1.0,
    'f1': 0.10714285714285715,
    'accuracy': 0.42528735632183906}


"""
Classifies: CHEBI:11750 3-sn-phosphatidyl-L-serine
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_3_sn_phosphatidyl_L_serine(smiles: str):
    """
    Determines if a molecule is a 3-sn-phosphatidyl-L-serine.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a 3-sn-phosphatidyl-L-serine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Check for phosphate group
    patt_phosphate = Chem.MolFromSmarts('[O,OH]-P(=O)([O,OH])-O')
    if not mol.HasSubstructMatch(patt_phosphate):
        return False, "Missing phosphate group"
        
    # Check for serine moiety
    patt_serine = Chem.MolFromSmarts('[NH2][CH]([CH2]OP(=O)([O,OH])[O,OH])C(=O)[O,OH]')
    if not mol.HasSubstructMatch(patt_serine):
        return False, "Missing serine moiety"
        
    # Check for glycerol backbone with 2 ester groups
    patt_glycerol = Chem.MolFromSmarts('[CH2]OC(=O)[*]-[CH]([CH2]OP(=O)([O,OH])[O,OH])OC(=O)[*]')
    if not mol.HasSubstructMatch(patt_glycerol):
        return False, "Missing glycerol backbone with ester groups"
        
    # Check stereochemistry at the 2-position of glycerol (sn-3)
    matches = mol.GetSubstructMatches(patt_glycerol)
    if matches:
        for match in matches:
            # Get the chiral center atom
            chiral_atom = mol.GetAtomWithIdx(match[2])  # Index of the CH in the glycerol backbone
            if chiral_atom.GetChiralTag() == Chem.ChiralType.CHI_UNSPECIFIED:
                return False, "Unspecified stereochemistry at glycerol 2-position"
    else:
        return False, "Could not verify stereochemistry"
        
    # Check stereochemistry of serine (L-configuration)
    matches = mol.GetSubstructMatches(patt_serine)
    if matches:
        for match in matches:
            # Get the chiral center atom
            chiral_atom = mol.GetAtomWithIdx(match[1])  # Index of the CH in serine
            if chiral_atom.GetChiralTag() == Chem.ChiralType.CHI_UNSPECIFIED:
                return False, "Unspecified stereochemistry at serine alpha carbon"
    else:
        return False, "Could not verify serine stereochemistry"

    return True, "Valid 3-sn-phosphatidyl-L-serine structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:11750',
                          'name': '3-sn-phosphatidyl-L-serine',
                          'definition': 'A 3-sn-glycerophosphoserine compound '
                                        'having acyl substituents at the 1- '
                                        'and 2-hydroxy positions.',
                          'parents': ['CHEBI:18303']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183903,
    'num_false_negatives': 8,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999565006987076}


"""
Classifies: CHEBI:17950 beta-D-galactosyl-(1->4)-beta-D-glucosyl-(1<->1)-N-acylsphingosine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_beta_D_galactosyl__1__4__beta_D_glucosyl__1___1__N_acylsphingosine(smiles: str):
    """
    Determines if a molecule is a beta-D-galactosyl-(1->4)-beta-D-glucosyl-(1<->1)-N-acylsphingosine,
    which is a diosylceramide having beta-D-galactosyl-(1->4)-beta-D-glucose as the disaccharide component.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a beta-D-galactosyl-(1->4)-beta-D-glucosyl-(1<->1)-N-acylsphingosine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find the amide nitrogen atom
    amide_n = None
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'N' and atom.GetTotalNumHs() == 0:
            amide_n = atom
            break
    if amide_n is None:
        return False, "No amide nitrogen found"

    # Check the acyl chain
    acyl_atoms = [amide_n] + list(amide_n.GetNeighbors())
    acyl_chain = Chem.Mol(Chem.PathToSubmol(mol, [a.GetIdx() for a in acyl_atoms]))
    if not is_acyl_chain(acyl_chain):
        return False, "Acyl chain is not long enough or contains unsaturated bonds"

    # Check the sphingosine base
    sphingosine_atoms = [atom for atom in mol.GetAtoms() if atom not in acyl_atoms]
    sphingosine = Chem.Mol(Chem.PathToSubmol(mol, [a.GetIdx() for a in sphingosine_atoms]))
    if not is_sphingosine_base(sphingosine):
        return False, "Sphingosine base is not valid"

    # Check the disaccharide component
    disaccharide = find_disaccharide(sphingosine)
    if disaccharide is None:
        return False, "No valid disaccharide component found"
    if not is_beta_D_galactosyl__1__4__beta_D_glucosyl(disaccharide):
        return False, "Disaccharide component is not beta-D-galactosyl-(1->4)-beta-D-glucosyl"

    return True, "The molecule is a valid beta-D-galactosyl-(1->4)-beta-D-glucosyl-(1<->1)-N-acylsphingosine"

def is_acyl_chain(mol):
    """Checks if the given molecule is a valid acyl chain."""
    # Check if the molecule contains only C and H atoms
    if not all(atom.GetSymbol() in ['C', 'H'] for atom in mol.GetAtoms()):
        return False

    # Check if the chain is linear and fully saturated
    conf = mol.GetConformer()
    for bond in mol.GetBonds():
        if bond.GetBondType() != Chem.BondType.SINGLE:
            return False
        begin = conf.GetAtomPosition(bond.GetBeginAtomIdx())
        end = conf.GetAtomPosition(bond.GetEndAtomIdx())
        if not is_linear(begin, end):
            return False

    return True

def is_linear(p1, p2, tol=0.01):
    """Checks if two points are approximately linear."""
    dx = p2.x - p1.x
    dy = p2.y - p1.y
    dz = p2.z - p1.z
    return abs(dx) < tol or abs(dy) < tol or abs(dz) < tol

def is_sphingosine_base(mol):
    """Checks if the given molecule is a valid sphingosine base."""
    # TODO: Implement this function
    return True

def find_disaccharide(mol):
    """Finds the disaccharide component in the given molecule."""
    # TODO: Implement this function
    return None

def is_beta_D_galactosyl__1__4__beta_D_glucosyl(mol):
    """Checks if the given molecule is beta-D-galactosyl-(1->4)-beta-D-glucosyl."""
    # TODO: Implement this function
    return True


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17950',
                          'name': 'beta-D-galactosyl-(1->4)-beta-D-glucosyl-(1<->1)-N-acylsphingosine',
                          'definition': 'A diosylceramide having '
                                        'beta-D-galactosyl-(1->4)-beta-D-glucose '
                                        'as the disaccharide component.',
                          'parents': ['CHEBI:63415', 'CHEBI:79208']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Range Error\n'
             '\tidx\n'
             '\tViolation occurred on line 329 in file '
             'Code/GraphMol/ROMol.cpp\n'
             '\tFailed Expression: 58 < 58\n'
             '\tRDKIT: 2024.03.6\n'
             '\tBOOST: 1_85\n',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:24406 glycosylglycerol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_glycosylglycerol(smiles: str):
    """
    Determines if a molecule is a glycosylglycerol - a glycoside formed by condensation 
    of glycerol with a glycosyl group.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a glycosylglycerol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of glycerol moiety (C-C-C backbone with 3 oxygens)
    glycerol_pattern = Chem.MolFromSmarts("[OX2H1,OX2R]-[CH2]-[CH1](-[OX2H1,OX2R])-[CH2]-[OX2H1,OX2R]")
    if not mol.HasSubstructMatch(glycerol_pattern):
        return False, "No glycerol moiety found"

    # Check for presence of pyranose sugar ring
    sugar_pattern = Chem.MolFromSmarts("O1[CH1][CH1][CH1][CH1][CH1]1")
    if not mol.HasSubstructMatch(sugar_pattern):
        return False, "No pyranose sugar ring found"

    # Check for glycosidic linkage between sugar and glycerol
    glycosidic_pattern = Chem.MolFromSmarts("O1[CH1][CH1][CH1][CH1][CH1]1-[O]-[CH2,CH1]-[CH1,CH2]-[CH2]")
    if not mol.HasSubstructMatch(glycosidic_pattern):
        return False, "No glycosidic linkage between sugar and glycerol found"

    # Count hydroxy groups on sugar ring
    sugar_hydroxy_pattern = Chem.MolFromSmarts("O1[CH1][CH1][CH1][CH1][CH1]1-[OX2H1]")
    sugar_hydroxy_count = len(mol.GetSubstructMatches(sugar_hydroxy_pattern))
    
    if sugar_hydroxy_count < 3:
        return False, "Sugar ring does not have enough hydroxyl groups"

    # All criteria met
    return True, "Molecule contains glycosyl group linked to glycerol backbone via glycosidic bond"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24406',
                          'name': 'glycosylglycerol',
                          'definition': 'A glycoside resulting from the '
                                        'condensation of one of the hydroxy '
                                        'groups of glycerol with a glycosyl '
                                        'group.',
                          'parents': ['CHEBI:24400', 'CHEBI:36307']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183906,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999891249972812}


"""
Classifies: CHEBI:17408 monoacylglycerol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_monoacylglycerol(smiles: str):
    """
    Determines if a molecule is a monoacylglycerol (MAG).
    A MAG has one acyl group and two hydroxyl groups attached to a glycerol backbone.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a monoacylglycerol, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Define SMARTS patterns
    glycerol_backbone = Chem.MolFromSmarts("[CH2][CH]([OH,OR])[CH2][OH,OR]")
    ester_pattern = Chem.MolFromSmarts("[CX3](=[OX1])[OX2][C]")
    oh_pattern = Chem.MolFromSmarts("[OH]")
    
    # Check for glycerol backbone
    if not mol.HasSubstructMatch(glycerol_backbone):
        return False, "No glycerol backbone found"

    # Count ester groups
    ester_matches = len(mol.GetSubstructMatches(ester_pattern))
    if ester_matches != 1:
        return False, f"Found {ester_matches} ester groups, expected 1"

    # Count hydroxyl groups
    oh_matches = len(mol.GetSubstructMatches(oh_pattern))
    if oh_matches != 2:
        return False, f"Found {oh_matches} hydroxyl groups, expected 2"

    # Check for proper connectivity
    # Find glycerol carbons and ester attachment
    glycerol_matches = mol.GetSubstructMatches(glycerol_backbone)
    ester_matches = mol.GetSubstructMatches(ester_pattern)
    
    if not glycerol_matches or not ester_matches:
        return False, "Required structural elements not found"
        
    glycerol_atoms = set(glycerol_matches[0])
    ester_atoms = set(ester_matches[0])
    
    # Check if ester is connected to glycerol backbone
    ester_oxygen = ester_atoms.intersection(glycerol_atoms)
    if not ester_oxygen:
        return False, "Ester not properly connected to glycerol backbone"

    return True, "Valid monoacylglycerol structure found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17408',
                          'name': 'monoacylglycerol',
                          'definition': 'A glyceride in which any one of the R '
                                        'groups (position not specified) is an '
                                        'acyl group while the remaining two R '
                                        'groups can be either H or alkyl '
                                        'groups.',
                          'parents': ['CHEBI:47778']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               'False negatives: '
               "[('CCCCC\\\\C=C/C\\\\C=C/CCCCCCCC(=O)OC(CO)CO', 'No glycerol "
               "backbone found'), "
               "('O(C(=O)CCCCCCCCCCCCCCCCCCCCC)C[C@@H](O)CO', 'Ester group not "
               "properly connected to glycerol backbone'), "
               "('C(O)C(OC(*)=O)CO', 'No glycerol backbone found'), "
               "('CCCCCCCCCCCCCCCCCC(=O)OC(CO)CO', 'No glycerol backbone "
               "found'), ('CC(C)CCCCC=CCCCCCCCC(=O)OCC(CO)O', 'Ester group not "
               "properly connected to glycerol backbone'), "
               "('C(CCCCCCCCCCCC(OCC(CO)O)=O)CCCCCCC', 'Ester group not "
               "properly connected to glycerol backbone'), "
               "('CCCCC\\\\C=C\\\\C=C1/[C@@H](C\\\\C=C/CCCC(=O)OC(CO)CO)C=CC1=O', "
               "'No glycerol backbone found'), ('C(O)C(O)COC(=O)*', 'Ester "
               "group not properly connected to glycerol backbone'), "
               "('CCCCCCCCCCCC(=O)OC[C@@H](O)CO', 'Ester group not properly "
               "connected to glycerol backbone'), "
               "('O(C(=O)CCCCCCCCCCCCCCCC)CC(O)CO', 'Ester group not properly "
               "connected to glycerol backbone'), "
               "('O(C[C@@H](O)CO)C(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC', "
               "'Ester group not properly connected to glycerol backbone'), "
               "('O(C(=O)CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCC)C(CO)CO', "
               "'No glycerol backbone found'), "
               "('O=C1OC(/C(=C/C(=O)OC[C@H](O)CO)/C)=CC(=C1)OC', 'Ester group "
               "not properly connected to glycerol backbone'), "
               "('O=C(OCC(O)CO)CCCCCCCCCCCCCCC(CC)C', 'Ester group not "
               "properly connected to glycerol backbone'), "
               "('OC(CO)COC(=O)CCCCCCC/C=C/CCCCCCCC', 'Ester group not "
               "properly connected to glycerol backbone'), "
               "('CCCCCCCCCCCCCC(=O)OC[C@H](O)CO', 'Ester group not properly "
               "connected to glycerol backbone'), "
               "('C(CCCCCCCCCCCC)CCCCCCCCC(OCC(O)CO)=O', 'Ester group not "
               "properly connected to glycerol backbone'), "
               "('O(C(=O)CCCCCCCCCCC/C=C\\\\CCCCCCCC)C(CO)CO', 'No glycerol "
               "backbone found')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 12,
    'num_false_positives': 12,
    'num_true_negatives': 183768,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.5,
    'recall': 0.6666666666666666,
    'f1': 0.5714285714285715,
    'accuracy': 0.9999020663989815}


"""
Classifies: CHEBI:17176 long-chain fatty aldehyde
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from collections import deque

def is_long_chain_fatty_aldehyde(smiles: str):
    """
    Determines if a molecule is a long-chain fatty aldehyde (C13-C22).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a long-chain fatty aldehyde, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for aldehyde group (C=O with H)
    aldehyde_pattern = Chem.MolFromSmarts('[CH1](=O)')
    if not mol.HasSubstructMatch(aldehyde_pattern):
        return False, "No aldehyde group found"

    # Count number of aldehyde groups
    matches = mol.GetSubstructMatches(aldehyde_pattern)
    if len(matches) > 1:
        return False, "Multiple aldehyde groups found"

    # Get the aldehyde carbon
    aldehyde_carbon = matches[0][0]

    # Function to check if atom is part of an aromatic ring
    def is_in_aromatic_ring(atom_idx):
        atom = mol.GetAtomWithIdx(atom_idx)
        return atom.GetIsAromatic()

    # Find the longest carbon chain containing the aldehyde carbon
    def find_longest_chain(start_idx):
        visited = {start_idx}
        queue = deque([(start_idx, [start_idx])])
        longest_path = []
        
        while queue:
            current_idx, current_path = queue.popleft()
            current_atom = mol.GetAtomWithIdx(current_idx)
            
            if len(current_path) > len(longest_path):
                longest_path = current_path
                
            for neighbor in current_atom.GetNeighbors():
                neighbor_idx = neighbor.GetIdx()
                if (neighbor_idx not in visited and 
                    neighbor.GetSymbol() == 'C'):
                    visited.add(neighbor_idx)
                    queue.append((neighbor_idx, current_path + [neighbor_idx]))
                    
        return longest_path

    longest_chain = find_longest_chain(aldehyde_carbon)
    chain_length = len(longest_chain)
    
    # Check chain length
    if chain_length < 13 or chain_length > 22:
        return False, f"Carbon chain length C{chain_length} outside C13-C22 range"

    # Check if the molecule is primarily a straight chain
    total_carbons = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    branch_carbons = total_carbons - chain_length
    if branch_carbons > 2:  # Allow minimal branching
        return False, "Structure too branched for a fatty aldehyde"
        
    # Check for cyclic structures
    if mol.GetRingInfo().NumRings() > 0:
        return False, "Contains cyclic structures"

    # Check for carboxylic acid groups
    carboxyl_pattern = Chem.MolFromSmarts('[CX3](=O)[OX2H1]')
    if mol.HasSubstructMatch(carboxyl_pattern):
        return False, "Contains carboxylic acid group"

    return True, f"Long-chain fatty aldehyde with {chain_length} carbons"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17176',
                          'name': 'long-chain fatty aldehyde',
                          'definition': 'A fatty aldehyde that has a chain '
                                        'length ranging from C13 to C22.',
                          'parents': ['CHEBI:35746']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.038461538461538464 is too low.\n'
               "True positives: [('[H]C(=O)CCCCCC\\\\C=C/CCCCCCCC', "
               "'Long-chain fatty aldehyde with 17 carbons'), "
               "('C(CCCCCCCCCC(C(=O)[H])O)CCCC', 'Long-chain fatty aldehyde "
               "with 16 carbons')]\n"
               "False positives: [('O=CCCCCCCCCCCCCCCCCCC', 'Long-chain fatty "
               "aldehyde with 19 carbons'), "
               "('O=CCC\\\\C=C\\\\C=C\\\\CCC\\\\C=C\\\\CCCC', 'Long-chain "
               "fatty aldehyde with 16 carbons'), ('O=CCCCC/C=C/CCC/C=C/CCCC', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('O=C(CCCCCCCCCCC)CC=O', 'Long-chain fatty aldehyde with 14 "
               "carbons'), ('O=CCCCCCCCC/C=C/CCCCC', 'Long-chain fatty "
               "aldehyde with 16 carbons'), "
               "('O=CCC(CCC[C@@H](CCC[C@@H](CCCC(C)C)C)C)C', 'Long-chain fatty "
               "aldehyde with 16 carbons'), "
               "('[H]C(=O)\\\\C=C\\\\CCCCCCCCCC(C)C', 'Long-chain fatty "
               "aldehyde with 14 carbons'), ('O=CCCCCCCC\\\\C=C\\\\CCCCCCCC', "
               "'Long-chain fatty aldehyde with 18 carbons'), "
               "('O=CCCCCCCCC/C=C/C=C/C=C/C', 'Long-chain fatty aldehyde with "
               "16 carbons'), ('[H]C(=O)CCCC([H])=CCC=C([H])CCCCC', "
               "'Long-chain fatty aldehyde with 14 carbons'), "
               "('O=C(C#CC#CCCCCCCCCCC)CC=O', 'Long-chain fatty aldehyde with "
               "17 carbons'), "
               "('[H]C(=O)C(\\\\C)=C\\\\C=C\\\\C(C)=C\\\\C=C\\\\C=C(C)\\\\C=C\\\\C=C(/C)C([H])=O', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('O=CCCCCCCC/C=C/C\\\\C=C\\\\CCCCC', 'Long-chain fatty "
               "aldehyde with 18 carbons'), ('O=CCCCCCCC/C=C/C=C/CCCC', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('O=C(CCCCC/C=C\\\\CCCCCC)CC=O', 'Long-chain fatty aldehyde "
               "with 16 carbons'), ('O=CCCCCCCC=CCC=CCCCCC', 'Long-chain fatty "
               "aldehyde with 17 carbons'), ('CCCCCCCCCCCCCCCCCC=O', "
               "'Long-chain fatty aldehyde with 18 carbons'), "
               "('C(\\\\C=C\\\\C([H])=O)(=C/C=C/C(=C/C=C/C=C(/C=C/C=C(/C=C/C(=O)[H])\\\\C)\\\\C)/C)/C', "
               "'Long-chain fatty aldehyde with 20 carbons'), "
               "('O=CCCCCCCCCCC#C/C=C\\\\CC', 'Long-chain fatty aldehyde with "
               "16 carbons'), ('O=CCCCCCCCCC/C=C/CC', 'Long-chain fatty "
               "aldehyde with 14 carbons'), "
               "('O=C/C=C/C=C/C=C/C=C/C=C/C=C/C=C/C=C/C=C/C', 'Long-chain "
               "fatty aldehyde with 20 carbons'), "
               "('O=CCCCCCCCCC/C=C/C=C\\\\CC', 'Long-chain fatty aldehyde with "
               "16 carbons'), ('O=C/C=C/C=C/C=C/C=C/C=C/C=C/C=O', 'Long-chain "
               "fatty aldehyde with 14 carbons'), "
               "('O=CCC[C@H](CC/C=C\\\\CC/C=C\\\\CCCCC)C', 'Long-chain fatty "
               "aldehyde with 17 carbons'), ('IC(CCCCCCCCCCCCCC)C=O', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('C(CCCCCCCCCC(=O)[H])CCCCC([O-])=O', 'Long-chain fatty "
               "aldehyde with 16 carbons'), "
               "('O=C/C=C/C=C/C=C/C=C/C=C/C=C/C/C=C\\\\C', 'Long-chain fatty "
               "aldehyde with 17 carbons'), ('[H]C(=O)C(C)CCCCCCCCCCCCC', "
               "'Long-chain fatty aldehyde with 15 carbons'), "
               "('O=C/C=C\\\\C(CCCCCCCCCC)=C', 'Long-chain fatty aldehyde with "
               "14 carbons'), ('O=CCCCCCC/C=C/C/C=C\\\\C/C=C/CC', 'Long-chain "
               "fatty aldehyde with 17 carbons'), "
               "('O=C/C=C/C=C/C=C/C=C/C=C/[C@H](O)C/C=C\\\\C', 'Long-chain "
               "fatty aldehyde with 16 carbons'), "
               "('[O-]C(=O)CCCCCCCCCCCCCCCCCCCCC=O', 'Long-chain fatty "
               "aldehyde with 22 carbons'), ('CC(C)CCCC(C)CCCC(C)CCCC(C)C=O', "
               "'Long-chain fatty aldehyde with 15 carbons'), "
               "('O=CCCCCC/C=C\\\\CCCCCC', 'Long-chain fatty aldehyde with 14 "
               "carbons'), "
               "('C(/C=C/C(=C/C=C/C=C(/C=C/C=C(/C=C/C=C(\\\\CCC=C(C)C)/C)\\\\C)\\\\C)/C)([H])=O', "
               "'Long-chain fatty aldehyde with 22 carbons'), "
               "('CC(CC/C=C(\\\\CC/C=C(\\\\CCC(=O)[H])/C)/C)=O', 'Long-chain "
               "fatty aldehyde with 13 carbons'), "
               "('O=CCCCCCCCC\\\\C=C/C=C\\\\CCC', 'Long-chain fatty aldehyde "
               "with 16 carbons'), ('BrC(CCCCCCCCCCCCCC)C=O', 'Long-chain "
               "fatty aldehyde with 16 carbons'), "
               "('O=C/C=C\\\\C=C\\\\CCCCCCCCCC', 'Long-chain fatty aldehyde "
               "with 15 carbons'), ('C(CCCCCCCCCC(=O)[H])CCCCC(O)=O', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('O=CCCCCCCCCCCCCCCCCCCC', 'Long-chain fatty aldehyde with 20 "
               "carbons'), ('O=CCCCCC\\\\C=C\\\\CCCCCC', 'Long-chain fatty "
               "aldehyde with 14 carbons'), ('O=CCC\\\\C=C\\\\C=C/CCCCCCCCC', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('O=C/C=C(/CC/C=C(/CC/C=C(/CCC=C(C)C)\\\\C)\\\\C)\\\\C', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('O=C\\\\C=C\\\\C=C\\\\C/C=C\\\\CCCCC', 'Long-chain fatty "
               "aldehyde with 13 carbons'), ('O=CCCCCCCCCC/C=C\\\\C=C/CC', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('OC(=O)CCCCCCC/C=C/C=C/C=O', 'Long-chain fatty aldehyde with "
               "13 carbons'), ('O=C\\\\C=C\\\\C=C\\\\CCCCCCCCCC', 'Long-chain "
               "fatty aldehyde with 15 carbons'), "
               "('O=CCCCCC/C=C\\\\CC/C=C\\\\C=C\\\\CC', 'Long-chain fatty "
               "aldehyde with 16 carbons'), ('O=CCCCCCCCCC/C=C\\\\CCCCCCCC', "
               "'Long-chain fatty aldehyde with 20 carbons'), "
               "('[H]C(=O)\\\\C=C\\\\CCCCCCCCCCCCC', 'Long-chain fatty "
               "aldehyde with 16 carbons'), ('O=CCCC\\\\C=C\\\\CCCCCCCC', "
               "'Long-chain fatty aldehyde with 14 carbons'), "
               "('O=CCCCCCCC/C=C\\\\C=C\\\\CCCC', 'Long-chain fatty aldehyde "
               "with 16 carbons'), ('O=CCCCCCCC/C=C/C=C\\\\CCCC', 'Long-chain "
               "fatty aldehyde with 16 carbons'), "
               "('[H]C(=O)C([H])=C([H])CCCCCCCCCCCCC', 'Long-chain fatty "
               "aldehyde with 16 carbons'), "
               "('[H]C(=O)C=C([H])CCCCCCCC([H])=CCC', 'Long-chain fatty "
               "aldehyde with 14 carbons'), ('O=CCCCCCCCC/C=C/C=C/C=C\\\\C', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('O=CCCCCC/C=C\\\\CC\\\\C=C\\\\CCCC', 'Long-chain fatty "
               "aldehyde with 16 carbons'), "
               "('O=C\\\\C=C\\\\CCCCCCCCC/C=C\\\\CCCC', 'Long-chain fatty "
               "aldehyde with 18 carbons'), ('O=CCCCCCCC/C=C/CCCC', "
               "'Long-chain fatty aldehyde with 14 carbons'), "
               "('O=C\\\\C=C\\\\CCCCCCCCCCCCCCC', 'Long-chain fatty aldehyde "
               "with 18 carbons'), ('O=C/C=C\\\\CCCCCCCCCCCCC', 'Long-chain "
               "fatty aldehyde with 16 carbons'), ('O=CCCC/C=C\\\\CCCCCCCC', "
               "'Long-chain fatty aldehyde with 14 carbons'), "
               "('OC(=O)CCCCCCCCCCCCCCCCCCCCC=O', 'Long-chain fatty aldehyde "
               "with 22 carbons'), ('O=CCCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC', "
               "'Long-chain fatty aldehyde with 18 carbons'), "
               "('O=C(CCCCCCCCCC=C)CC=O', 'Long-chain fatty aldehyde with 14 "
               "carbons'), ('O=CCC\\\\C=C\\\\CCC/C=C\\\\CCCC', 'Long-chain "
               "fatty aldehyde with 14 carbons'), ('O=CCCCCC/C=C\\\\CCCCCCCC', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('O=CC(CCCCCCCCCCCCCCCCCCC)CCCCCCCCC', 'Long-chain fatty "
               "aldehyde with 21 carbons'), ('O=C\\\\C=C\\\\CCCCCCCCCCC', "
               "'Long-chain fatty aldehyde with 14 carbons'), "
               "('O=CCC/C=C\\\\CCCCCCCC', 'Long-chain fatty aldehyde with 13 "
               "carbons'), ('O=CCCCCCCC/C=C\\\\C=C/CCCC', 'Long-chain fatty "
               "aldehyde with 16 carbons'), ('O=CCCCC/C=C/C=C/C=C/C=C/CC', "
               "'Long-chain fatty aldehyde with 15 carbons'), "
               "('O=CCCCCCCCCCC/C=C\\\\CCC', 'Long-chain fatty aldehyde with "
               "16 carbons'), ('O=CCCCCCCCCC/C=C\\\\C=C\\\\CC', 'Long-chain "
               "fatty aldehyde with 16 carbons'), "
               "('C(=O)([H])/C=C/CCCCCCCC/C=C\\\\CCC', 'Long-chain fatty "
               "aldehyde with 16 carbons'), ('O=CCCCCCCCC/C=C\\\\CCCCCCCC', "
               "'Long-chain fatty aldehyde with 19 carbons'), "
               "('O=C\\\\C=C\\\\CCCCCCCCCC', 'Long-chain fatty aldehyde with "
               "13 carbons'), ('O=CCCCCC\\\\C=C\\\\CCCCCCCC', 'Long-chain "
               "fatty aldehyde with 16 carbons'), "
               "('O=CCCCCCC\\\\C=C\\\\CCCCC(CC)C', 'Long-chain fatty aldehyde "
               "with 16 carbons'), ('O=C\\\\C=C\\\\C=C\\\\C\\\\C=C\\\\CCCCC', "
               "'Long-chain fatty aldehyde with 13 carbons'), "
               "('O=CCCCCCCCC/C=C\\\\CCCCC', 'Long-chain fatty aldehyde with "
               "16 carbons'), ('O=CCCCCCCCCC/C=C/C=C/CC', 'Long-chain fatty "
               "aldehyde with 16 carbons'), ('O=CCCCC/C=C/CCC/C=C\\\\CCCC', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('O=CCC(C(C/C=C(/CC/C=C(\\\\CC)/C)\\\\C)C)C', 'Long-chain "
               "fatty aldehyde with 13 carbons'), ('O=CCCCCCCC/C=C/CCCCCC', "
               "'Long-chain fatty aldehyde with 16 carbons'), "
               "('O=CCCCCCCCCCC#CCCCC', 'Long-chain fatty aldehyde with 16 "
               "carbons'), "
               "('[O-]C(=O)CCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC=O', "
               "'Long-chain fatty aldehyde with 20 carbons'), "
               "('O=CCC\\\\C=C\\\\C=C\\\\CCC/C=C\\\\CCCC', 'Long-chain fatty "
               "aldehyde with 16 carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCC=O', 'Long-chain fatty "
               "aldehyde with 17 carbons'), ('O=CCCCCCCCCC/C=C\\\\C=C', "
               "'Long-chain fatty aldehyde with 14 carbons'), "
               "('O=CCCCCCCC/C=C\\\\CCCC', 'Long-chain fatty aldehyde with 14 "
               "carbons'), ('O=CC(CCCC(CCCC(CCCC(C)C)C)C)=C', 'Long-chain "
               "fatty aldehyde with 15 carbons'), "
               "('O=CCCCCCC\\\\C=C\\\\CCCC[C@@H](CC)C', 'Long-chain fatty "
               "aldehyde with 16 carbons'), ('O=CCCCCCCCCC/C=C\\\\CC', "
               "'Long-chain fatty aldehyde with 14 carbons'), "
               "('C(\\\\C=C/C=C/CCC)CCCCCCCC([H])=O', 'Long-chain fatty "
               "aldehyde with 16 carbons'), "
               "('C(\\\\CCCCCCCC(=O)O)=C\\\\CCCCCCCC(=O)[H]', 'Long-chain "
               "fatty aldehyde with 18 carbons'), "
               "('O=CCCCCCCCC/C=C/C=C\\\\CCC', 'Long-chain fatty aldehyde with "
               "16 carbons'), ('O=CCCCCCCCCC/C=C/C=C', 'Long-chain fatty "
               "aldehyde with 14 carbons'), ('O=CCCCCCCC/C=C/C/C=C/C/C=C/CC', "
               "'Long-chain fatty aldehyde with 18 carbons')]\n"
               'False negatives: []',
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 96,
    'num_true_negatives': 183815,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02040816326530612,
    'recall': 1.0,
    'f1': 0.039999999999999994,
    'accuracy': 0.9994780140609962}


"""
Classifies: CHEBI:23453 cycloalkane
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_cycloalkane(smiles: str):
    """
    Determines if a molecule is a cycloalkane (saturated monocyclic hydrocarbon).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a cycloalkane, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if molecule contains only carbon and hydrogen
    atoms = mol.GetAtoms()
    if not all(atom.GetSymbol() in ['C', 'H'] for atom in atoms):
        return False, "Contains non C/H atoms"

    # Get ring information
    rings = mol.GetRingInfo()
    ring_count = rings.NumRings()
    
    # Must have exactly one ring
    if ring_count != 1:
        return False, f"Has {ring_count} rings, must have exactly 1 ring"

    # Get the ring atoms
    ring = rings.AtomRings()[0]
    ring_atoms = [mol.GetAtomWithIdx(i) for i in ring]
    
    # Check if any ring atoms are aromatic or unsaturated
    for atom in ring_atoms:
        if atom.GetIsAromatic():
            return False, "Contains aromatic ring"
        if any(bond.GetBondTypeAsDouble() != 1.0 for bond in atom.GetBonds()):
            return False, "Contains unsaturated bonds"

    # Count carbons in ring and side chains
    ring_size = len(ring)
    total_carbons = sum(1 for atom in atoms if atom.GetSymbol() == 'C')
    side_chain_carbons = total_carbons - ring_size
    
    if side_chain_carbons > 0:
        return True, f"{ring_size}-membered cycloalkane with {side_chain_carbons} carbons in side chains"
    else:
        return True, f"{ring_size}-membered cycloalkane"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23453',
                          'name': 'cycloalkane',
                          'definition': 'Saturated monocyclic hydrocarbons '
                                        '(with or without side chains).',
                          'parents': ['CHEBI:33654', 'CHEBI:33664']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 15,
    'num_true_negatives': 183881,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.16666666666666666,
    'recall': 1.0,
    'f1': 0.2857142857142857,
    'accuracy': 0.9999184334879472}


"""
Classifies: CHEBI:26400 purine ribonucleotide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect
import re

def is_purine_ribonucleotide(smiles: str):
    """
    Determines if a molecule is a purine ribonucleotide.
    A purine ribonucleotide must have:
    1. A purine base (adenine or guanine derivatives)
    2. A ribose sugar
    3. At least one phosphate group
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a purine ribonucleotide, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for phosphate group
    if not 'P' in [atom.GetSymbol() for atom in mol.GetAtoms()]:
        return False, "No phosphate group found"
    
    # Check for purine scaffold
    purine_pattern = Chem.MolFromSmarts('[#7]1[#6]=N[#6]2[#6](=[#7])[#7][#6]=[#7][#6]12')
    if not mol.HasSubstructMatch(purine_pattern):
        return False, "No purine base found"
        
    # Check for ribose sugar connected to purine
    ribose_pattern = Chem.MolFromSmarts('[C@@H]1O[C@H](CO)[C@@H](O)[C@H]1O')
    if not mol.HasSubstructMatch(ribose_pattern):
        return False, "No ribose sugar found"
        
    # Check connection between purine and ribose
    purine_ribose_pattern = Chem.MolFromSmarts('[#7]1[#6]=N[#6]2[#6](=[#7])[#7][#6]=[#7][#6]12-[C@@H]1O[C@H](CO)[C@@H](O)[C@H]1O')
    if not mol.HasSubstructMatch(purine_ribose_pattern):
        return False, "Purine not properly connected to ribose"

    # Determine purine base type
    if mol.HasSubstructMatch(Chem.MolFromSmarts('Nc1ncnc2[nH]cnc12')):
        base = "adenine"
    elif mol.HasSubstructMatch(Chem.MolFromSmarts('O=c1[nH]c(N)nc2[nH]cnc12')):
        base = "guanine"
    else:
        base = "modified purine"

    # Count phosphate groups
    phosphate_pattern = Chem.MolFromSmarts('OP(=O)(O)O')
    num_phosphates = len(mol.GetSubstructMatches(phosphate_pattern))
    
    return True, f"Purine ribonucleotide with {base} base and {num_phosphates} phosphate group(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26400',
                          'name': 'purine ribonucleotide',
                          'definition': 'Any ribonucleotide that has a purine '
                                        'nucleobase.',
                          'parents': ['CHEBI:26395', 'CHEBI:26561']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183760,
    'num_false_negatives': 17,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999074965855357}


"""
Classifies: CHEBI:134362 homoallylic alcohol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_homoallylic_alcohol(smiles: str):
    """
    Determines if a molecule contains a homoallylic alcohol group.
    A homoallylic alcohol has a hydroxy group beta to a double bond.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains homoallylic alcohol, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Add explicit hydrogens
    mol = Chem.AddHs(mol)
    
    # SMARTS pattern for homoallylic alcohol:
    # [OH]-[CH2]-[CH2]-C=C
    # The pattern looks for:
    # - An OH group
    # - Connected to a carbon (beta carbon)
    # - Connected to another carbon (alpha carbon) 
    # - Connected to a carbon involved in a double bond
    pattern = Chem.MolFromSmarts('[OH]-[#6]-[#6]-[#6]=[#6]')
    
    if mol.HasSubstructMatch(pattern):
        # Find all matches
        matches = mol.GetSubstructMatches(pattern)
        
        # For each match, verify it's an aliphatic alcohol
        for match in matches:
            oh_atom = mol.GetAtomWithIdx(match[0])
            beta_carbon = mol.GetAtomWithIdx(match[1])
            alpha_carbon = mol.GetAtomWithIdx(match[2])
            double_bond_carbon = mol.GetAtomWithIdx(match[3])
            
            # Check if carbons are aliphatic (not aromatic)
            if not any(atom.GetIsAromatic() for atom in [beta_carbon, alpha_carbon, double_bond_carbon]):
                return True, "Contains homoallylic alcohol group: OH beta to double bond"
                
    return False, "No homoallylic alcohol group found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:134362',
                          'name': 'homoallylic alcohol',
                          'definition': 'An aliphatic alcohol where the '
                                        'hydroxy carbon is beta to a double '
                                        'bond.',
                          'parents': ['CHEBI:2571', 'CHEBI:78840']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 1707,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.9448123620309051}